/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef BOT_HH
#define BOT_HH

#include "IStrategyContext.hh"

// Vorwaertsdeklaration
class BaseStrategy;

class Bot : public IStrategyContext
{
  public:
    /// Konstruktor.
    Bot();

    /// Destruktor.
    /**
     * Zerstoert die gesetzte Strategie, auch wenn diese
     * ausserhalb generiert wurde.
     */
    virtual ~Bot();

    /// Setzt die Strategie fuer einen Bot.
    /**
     * Die Strategie bestimmt, wie der Bot sich verhaelt
     * bzw. wann er Angebote annimmt/ablehnt.
     */
    virtual void setStrategy( BaseStrategy* strategyP );

    /// Akzeptiert oder lehnt ein Angebot ab.
    /**
     * Diese Methode berechnet anhand der Strategie, ob der Wert
     * angenommen oder abgelehnt wird. Der angebotene Wert
     * muss zwischen 0 und 1000 liegen.
     * @param value Der angebotene Wert.
     * @return true, wenn das Angebot akzeptiert wurde, ansonsten false
     */
    bool acceptOffer( const int value );

    /// Gibt die gesammelten Punkte zurueck.
    int getPoints() const;

  private:
    /// Die aktuelle Strategie.
    BaseStrategy* mStrategyP;
  
    /// Gesammelte Punkte.
    int mPoints;
};

#endif // BASEBOT_HH
