/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef VARIABLESTRATEGY_HH
#define VARIABLESTRATEGY_HH

// Vorwaertsdeklaration.
class IStrategyContext;

#include "BaseStrategy.hh"

class VariableStrategy : public BaseStrategy
{
  public:
    /// Konstruktor.
    /**
     * @param contextP Das Objekt, welches die Strategie nutzt.
     */
    VariableStrategy( IStrategyContext* contextP );

    /// Copy Konstruktor.
    /**
     * @param object Das zu kopierende Objekt (ohne Strategie-Kontext).
     * @param contextP Das Objekt, welches die Strategie nutzt.
     */
    VariableStrategy( const VariableStrategy& object, IStrategyContext* contextP );

    /// Akzeptiert oder lehnt ein Angebot ab.
    /**
     * Diese Methode berechnet anhand des Wertes und der vorherigen
     * Entscheidungen, ob angenommen oder abgelehnt wird.
     * Der angebotene Wert muss zwischen 0 und 1000 liegen.
     * @param value Der angebotene Wert.
     * @return true, wenn das Angebot akzeptiert wurde, ansonsten false
     */
    bool acceptOffer( const int value );

    /// Erstellt eine Kopie des Elements.
    /**
     * Wenn man abgeleitete Klassen von einer Basis-Klasse hat
     * und nur mit einem Pointer auf die Basis-Klasse arbeitet,
     * lassen sich Objekte nicht so einfach per Zuweisung kopieren
     * da man ja die exakte Klassen benötigt. In dem Fall arbeitet
     * man mit einer virtuellen copy-Methode, welche in jeder
     * abgeleiteten Klasse implementiert werden muss.
     * @return Pointer auf die Kopie des Objekt.
     */
    virtual BaseStrategy* copy( IStrategyContext* contextP ) const;
    
private:
    /// Anzahl der nacheinander angenommenen Angebote.
    int mNumAcceptInRow;

    /// Anzahl der nacheinander abgelehnten Angebote.
    int mNumDeclineInRow;
};

#endif // VARIABLESTRATEGY_HH
